@@ -216,11 +216,16 @@ var _ = Describe("Expose", func() {
 				})
 			})
 			Context("With cluster-ip on an unknown vm", func() {
-				It("should fail", func() {
+				It("should fail on a vmi", func() {
 					cmd := tests.NewRepeatableVirtctlCommand(expose.COMMAND_EXPOSE, "vmi", unknownVM, "--name", "my-service",
 						"--port", "9999")
 					Expect(cmd()).NotTo(BeNil())
 				})
+				It("should fail on a vm", func() {
+					cmd := tests.NewRepeatableVirtctlCommand(expose.COMMAND_EXPOSE, "vm", unknownVM, "--name", "my-service",
+						"--port", "9999")
+					Expect(cmd()).NotTo(BeNil())
+				})
 			})
 			Context("With node-port service on a vm", func() {
 				It("should succeed", func() {
@@ -236,13 +241,6 @@ var _ = Describe("Expose", func() {
 					Expect(err()).To(BeNil())
 				})
 			})
-			Context("With cluster-ip on an unknown vm", func() {
-				It("should fail", func() {
-					cmd := tests.NewRepeatableVirtctlCommand(expose.COMMAND_EXPOSE, "vm", unknownVM, "--name", "my-service",
-						"--port", "9999")
-					Expect(cmd()).NotTo(BeNil())
-				})
-			})
 			Context("With cluster-ip on an vm replica set", func() {
 				It("should succeed", func() {
 					err := tests.NewRepeatableVirtctlCommand(expose.COMMAND_EXPOSE, "vmirs", vmName, "--name", "my-service",
