@@ -31,14 +31,17 @@ import (
 	"github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
 
+	"kubevirt.io/kubevirt/tests/util"
+
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	kubevirtv1 "kubevirt.io/client-go/api/v1"
 	v1 "kubevirt.io/client-go/api/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1alpha1"
+	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/kubevirt/tests"
 	"kubevirt.io/kubevirt/tests/console"
 	cd "kubevirt.io/kubevirt/tests/containerdisk"
@@ -68,7 +71,7 @@ var _ = SIGDescribe("Hotplug", func() {
 
 	BeforeEach(func() {
 		virtClient, err = kubecli.GetKubevirtClient()
-		tests.PanicOnError(err)
+		util.PanicOnError(err)
 
 		tests.BeforeTestCleanup()
 	})
@@ -81,13 +84,13 @@ var _ = SIGDescribe("Hotplug", func() {
 	createVirtualMachine := func(running bool, template *kubevirtv1.VirtualMachineInstance) *kubevirtv1.VirtualMachine {
 		By("Creating VirtualMachine")
 		vm := tests.NewRandomVirtualMachine(template, running)
-		newVM, err := virtClient.VirtualMachine(tests.NamespaceTestDefault).Create(vm)
+		newVM, err := virtClient.VirtualMachine(util.NamespaceTestDefault).Create(vm)
 		Expect(err).ToNot(HaveOccurred())
 		return newVM
 	}
 
 	deleteVirtualMachine := func(vm *kubevirtv1.VirtualMachine) error {
-		return virtClient.VirtualMachine(tests.NamespaceTestDefault).Delete(vm.Name, &metav1.DeleteOptions{})
+		return virtClient.VirtualMachine(util.NamespaceTestDefault).Delete(vm.Name, &metav1.DeleteOptions{})
 	}
 
 	getAddVolumeOptions := func(volumeName, bus string, volumeSource *kubevirtv1.HotplugVolumeSource) *kubevirtv1.AddVolumeOptions {
@@ -350,6 +353,23 @@ var _ = SIGDescribe("Hotplug", func() {
 		Expect(console.SafeExpectBatch(vmi, batch, 20)).To(Succeed())
 	}
 
+	verifyWriteReadData := func(vmi *kubevirtv1.VirtualMachineInstance, device string) {
+		dataFile := filepath.Join("/test", filepath.Base(device), "data/message")
+		batch := []expect.Batcher{
+			&expect.BSnd{S: fmt.Sprintf("echo '%s' > %s\n", vmi.UID, dataFile)},
+			&expect.BExp{R: console.PromptExpression},
+			&expect.BSnd{S: expectReturn},
+			&expect.BExp{R: console.RetValue("0")},
+			&expect.BSnd{S: fmt.Sprintf("cat %s\n", dataFile)},
+			&expect.BExp{R: string(vmi.UID)},
+			&expect.BSnd{S: "sync\n"},
+			&expect.BExp{R: console.PromptExpression},
+			&expect.BSnd{S: "sync\n"},
+			&expect.BExp{R: console.PromptExpression},
+		}
+		Expect(console.SafeExpectBatch(vmi, batch, 20)).To(Succeed())
+	}
+
 	verifyVolumePermanent := func(vmi *kubevirtv1.VirtualMachineInstance, volumeName string) {
 		updatedVMI, err := virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
@@ -441,7 +461,7 @@ var _ = SIGDescribe("Hotplug", func() {
 		template := libvmi.NewCirros()
 		vm := createVirtualMachine(true, template)
 		Eventually(func() bool {
-			vm, err := virtClient.VirtualMachine(tests.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
+			vm, err := virtClient.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			return vm.Status.Ready
 		}, 300*time.Second, 1*time.Second).Should(BeTrue())
@@ -457,6 +477,20 @@ var _ = SIGDescribe("Hotplug", func() {
 		return targets
 	}
 
+	verifySingleAttachmentPod := func(vmi *kubevirtv1.VirtualMachineInstance) {
+		podList, err := virtClient.CoreV1().Pods(vmi.Namespace).List(context.Background(), metav1.ListOptions{})
+		Expect(err).ToNot(HaveOccurred())
+		attachmentPodCount := 0
+		for _, pod := range podList.Items {
+			for _, ownerRef := range pod.GetOwnerReferences() {
+				if ownerRef.UID == vmi.GetUID() {
+					attachmentPodCount++
+				}
+			}
+		}
+		Expect(attachmentPodCount).To(Equal(1), "Number of attachment pods is not 1: %s", attachmentPodCount)
+	}
+
 	getVmiConsoleAndLogin := func(vmi *kubevirtv1.VirtualMachineInstance) {
 		By("Obtaining the serial console")
 		Expect(console.LoginToCirros(vmi)).To(Succeed())
@@ -468,8 +502,8 @@ var _ = SIGDescribe("Hotplug", func() {
 			accessMode = corev1.ReadWriteMany
 		}
 		By("Creating DataVolume")
-		dvBlock := tests.NewRandomBlankDataVolume(tests.NamespaceTestDefault, sc, "64Mi", accessMode, volumeMode)
-		_, err := virtClient.CdiClient().CdiV1alpha1().DataVolumes(dvBlock.Namespace).Create(context.Background(), dvBlock, metav1.CreateOptions{})
+		dvBlock := tests.NewRandomBlankDataVolume(util.NamespaceTestDefault, sc, "64Mi", accessMode, volumeMode)
+		_, err := virtClient.CdiClient().CdiV1beta1().DataVolumes(dvBlock.Namespace).Create(context.Background(), dvBlock, metav1.CreateOptions{})
 		Expect(err).To(BeNil())
 		tests.WaitForSuccessfulDataVolumeImport(dvBlock, 240)
 		return dvBlock
@@ -502,8 +536,8 @@ var _ = SIGDescribe("Hotplug", func() {
 
 			verifyVolumeAndDiskVMRemoved(vm, testNewVolume1, testNewVolume2)
 		},
-			table.Entry("[QUARANTINE]with DataVolume", addDVVolumeVM, removeVolumeVM),
-			table.Entry("[QUARANTINE]with PersistentVolume", addPVCVolumeVM, removeVolumeVM),
+			table.Entry("with DataVolume", addDVVolumeVM, removeVolumeVM),
+			table.Entry("with PersistentVolume", addPVCVolumeVM, removeVolumeVM),
 		)
 	})
 
@@ -533,13 +567,14 @@ var _ = SIGDescribe("Hotplug", func() {
 		})
 
 		table.DescribeTable("Should be able to add and use WFFC local storage", func(addVolumeFunc func(name, namespace, volumeName, claimName, bus string), removeVolumeFunc func(name, namespace, volumeName string)) {
+			tests.SkipIfNonRoot(virtClient, "root owned disk.img")
 			vmi, err := virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			tests.WaitForSuccessfulVMIStartWithTimeout(vmi, 240)
 			dvNames := make([]string, 0)
 			for i := 0; i < 3; i++ {
-				dv := tests.NewRandomBlankDataVolume(tests.NamespaceTestDefault, tests.Config.StorageClassLocal, "64Mi", corev1.ReadWriteOnce, corev1.PersistentVolumeFilesystem)
-				_, err := virtClient.CdiClient().CdiV1alpha1().DataVolumes(dv.Namespace).Create(context.TODO(), dv, metav1.CreateOptions{})
+				dv := tests.NewRandomBlankDataVolume(util.NamespaceTestDefault, tests.Config.StorageClassLocal, "64Mi", corev1.ReadWriteOnce, corev1.PersistentVolumeFilesystem)
+				_, err := virtClient.CdiClient().CdiV1beta1().DataVolumes(dv.Namespace).Create(context.TODO(), dv, metav1.CreateOptions{})
 				Expect(err).To(BeNil())
 				dvNames = append(dvNames, dv.Name)
 			}
@@ -555,6 +590,7 @@ var _ = SIGDescribe("Hotplug", func() {
 			verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, dvNames...)
 			getVmiConsoleAndLogin(vmi)
 			verifyHotplugAttachedAndUseable(vmi, dvNames)
+			verifySingleAttachmentPod(vmi)
 			for _, volumeName := range dvNames {
 				By("removing volume " + volumeName + " from VM")
 				removeVolumeFunc(vm.Name, vm.Namespace, volumeName)
@@ -568,7 +604,7 @@ var _ = SIGDescribe("Hotplug", func() {
 		)
 	})
 
-	Context("rook-ceph", func() {
+	Context("[rook-ceph]", func() {
 		Context("Online VM", func() {
 			var (
 				vm *kubevirtv1.VirtualMachine
@@ -607,7 +643,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				}
 				vm = createVirtualMachine(true, template)
 				Eventually(func() bool {
-					vm, err := virtClient.VirtualMachine(tests.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
+					vm, err := virtClient.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
 					Expect(err).ToNot(HaveOccurred())
 					return vm.Status.Ready
 				}, 300*time.Second, 1*time.Second).Should(BeTrue())
@@ -633,6 +669,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, "testvolume")
 				getVmiConsoleAndLogin(vmi)
 				targets := verifyHotplugAttachedAndUseable(vmi, []string{"testvolume"})
+				verifySingleAttachmentPod(vmi)
 				By("removing volume from VM")
 				removeVolumeFunc(vm.Name, vm.Namespace, "testvolume")
 				if !vmiOnly {
@@ -672,6 +709,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				verifyVolumeAndDiskVMIAdded(vmi, testVolumes...)
 				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, testVolumes...)
 				targets := verifyHotplugAttachedAndUseable(vmi, testVolumes)
+				verifySingleAttachmentPod(vmi)
 				for _, volumeName := range testVolumes {
 					By("removing volume " + volumeName + " from VM")
 					removeVolumeFunc(vm.Name, vm.Namespace, volumeName)
@@ -716,7 +754,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				verifyVolumeAndDiskVMIAdded(vmi, testVolumes[:len(testVolumes)-1]...)
 				waitForAttachmentPodToRun(vmi)
 				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, testVolumes[:len(testVolumes)-1]...)
-
+				verifySingleAttachmentPod(vmi)
 				By("removing volume sdc, with dv" + dvNames[2])
 				Eventually(func() string {
 					vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
@@ -760,7 +798,7 @@ var _ = SIGDescribe("Hotplug", func() {
 					}
 					return ""
 				}, 40*time.Second, 2*time.Second).Should(Equal("sde"))
-
+				verifySingleAttachmentPod(vmi)
 				for _, volumeName := range testVolumes {
 					By("removing volume from VM")
 					removeVolumeFunc(vm.Name, vm.Namespace, volumeName)
@@ -790,6 +828,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				Expect(err).ToNot(HaveOccurred())
 				verifyVolumeAndDiskVMIAdded(vmi, "testvolume")
 				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, "testvolume")
+				verifySingleAttachmentPod(vmi)
 
 				By("stopping VM")
 				vm = tests.StopVirtualMachine(vm)
@@ -838,7 +877,7 @@ var _ = SIGDescribe("Hotplug", func() {
 				for i := 0; i < 2; i++ {
 					verifyVolumeAccessible(vmi, targets[i])
 				}
-
+				verifySingleAttachmentPod(vmi)
 				removeVolumeVMI(vmi.Name, vmi.Namespace, "block")
 				removeVolumeVMI(vmi.Name, vmi.Namespace, "fs")
 
@@ -848,10 +887,19 @@ var _ = SIGDescribe("Hotplug", func() {
 			})
 		})
 
-		Context("VMI only", func() {
+		Context("VMI migration", func() {
 			var (
 				vmi *kubevirtv1.VirtualMachineInstance
 				sc  string
+
+				numberOfMigrations int
+				sourceNode         string
+				targetNode         string
+			)
+
+			const (
+				hotplugLabelKey   = "kubevirt-test-migration-with-hotplug-disks"
+				hotplugLabelValue = "true"
 			)
 
 			verifyIsMigratable := func(vmi *kubevirtv1.VirtualMachineInstance, expectedValue bool) {
@@ -876,53 +924,46 @@ var _ = SIGDescribe("Hotplug", func() {
 					Skip("Skip OCS tests when Ceph is not present")
 				}
 
-				vmi = tests.NewRandomFedoraVMI()
-				vmi = tests.RunVMIAndExpectLaunch(vmi, 240)
-			})
-
-			It("should mark VMI failed, if an attachment pod is deleted", func() {
-				volumeMode := corev1.PersistentVolumeFilesystem
-				addVolumeFunc := addDVVolumeVMI
-				dv := createDataVolumeAndWaitForImport(sc, volumeMode)
-
-				vmi, err := virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-				tests.WaitForSuccessfulVMIStartWithTimeout(vmi, 240)
-				By("Adding volume to running VMI")
-				addVolumeFunc(vmi.Name, vmi.Namespace, "testvolume", dv.Name, "scsi")
-				By("Verifying the volume and disk are in the VMI")
-				vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-				verifyVolumeAndDiskVMIAdded(vmi, "testvolume")
-				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, "testvolume")
-
-				podName := ""
-				vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-				for _, volumeStatus := range vmi.Status.VolumeStatus {
-					if volumeStatus.HotplugVolume != nil {
-						podName = volumeStatus.HotplugVolume.AttachPodName
-						break
+				// Workaround for the issue with CPU manager and runc prior to version v1.0.0:
+				// CPU manager periodically updates cgroup settings via the container runtime
+				// interface. Runc prior to version v1.0.0 drops all 'custom' cgroup device
+				// rules on 'update' and that causes a race with live migration when block volumes
+				// are hotplugged. Try to setup the test in a way so that the VMI is migrated to
+				// a node without CPU manager.
+				sourceNode = ""
+				targetNode = ""
+				for _, node := range util.GetAllSchedulableNodes(virtClient).Items {
+					labels := node.GetLabels()
+					if val, ok := labels[v1.CPUManager]; ok && val == "true" {
+						// Use a node with CPU manager as migration source
+						sourceNode = node.Name
+					} else {
+						// Use a node without CPU manager as migration target
+						targetNode = node.Name
 					}
 				}
-				Expect(podName).ToNot(BeEmpty())
-				By("Deleting attachment pod:" + podName)
-				zero := int64(0)
-				err = virtClient.CoreV1().Pods(vmi.Namespace).Delete(context.Background(), podName, metav1.DeleteOptions{
-					GracePeriodSeconds: &zero,
-				})
-				Expect(err).ToNot(HaveOccurred())
-				By("Verifying that VMI goes into failed state")
-				Eventually(func() bool {
-					vmi, err := virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
-					if err != nil {
-						return false
-					}
-					return vmi.Status.Phase == kubevirtv1.Failed
-				}, 90*time.Second, 1*time.Second).Should(BeTrue(), "VMI not in failed state")
+				if sourceNode == "" || targetNode == "" {
+					Skip("Two schedulable nodes are required for migration tests")
+				} else {
+					numberOfMigrations = 1
+				}
+				// Ensure the virt-launcher pod is scheduled on the chosen source node and then
+				// migrated to the proper target.
+				tests.AddLabelToNode(sourceNode, hotplugLabelKey, hotplugLabelValue)
+				vmi, _ = newVirtualMachineInstanceWithContainerDisk()
+				vmi.Spec.NodeSelector = map[string]string{hotplugLabelKey: hotplugLabelValue}
+				vmi = tests.RunVMIAndExpectLaunch(vmi, 240)
+				tests.AddLabelToNode(targetNode, hotplugLabelKey, hotplugLabelValue)
 			})
 
-			It("should mark VMI not migrateable, if a volume is attached", func() {
+			AfterEach(func() {
+				// Cleanup node labels
+				tests.RemoveLabelFromNode(sourceNode, hotplugLabelKey)
+				tests.RemoveLabelFromNode(targetNode, hotplugLabelKey)
+			})
+
+			It("should allow live migration with attached hotplug volumes", func() {
+				volumeName := "testvolume"
 				volumeMode := corev1.PersistentVolumeBlock
 				addVolumeFunc := addDVVolumeVMI
 				removeVolumeFunc := removeVolumeVMI
@@ -935,27 +976,55 @@ var _ = SIGDescribe("Hotplug", func() {
 				verifyIsMigratable(vmi, true)
 
 				By("Adding volume to running VMI")
-				addVolumeFunc(vmi.Name, vmi.Namespace, "testvolume", dv.Name, "scsi")
+				addVolumeFunc(vmi.Name, vmi.Namespace, volumeName, dv.Name, "scsi")
 				By("Verifying the volume and disk are in the VMI")
 				vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
-				verifyVolumeAndDiskVMIAdded(vmi, "testvolume")
-				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, "testvolume")
+				verifyVolumeAndDiskVMIAdded(vmi, volumeName)
+				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, volumeName)
 
-				By("Verifying the VMI is not migrateable")
-				verifyIsMigratable(vmi, false)
+				By("Verifying the VMI is still migrateable")
+				verifyIsMigratable(vmi, true)
+
+				By("Verifying the volume is attached and usable")
+				getVmiConsoleAndLogin(vmi)
+				targets := verifyHotplugAttachedAndUseable(vmi, []string{volumeName})
+				Expect(len(targets) == 1).To(BeTrue())
+
+				By("Starting the migration multiple times")
+				for i := 0; i < numberOfMigrations; i++ {
+					vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+					sourceAttachmentPods := []string{}
+					for _, volumeStatus := range vmi.Status.VolumeStatus {
+						if volumeStatus.HotplugVolume != nil {
+							sourceAttachmentPods = append(sourceAttachmentPods, volumeStatus.HotplugVolume.AttachPodName)
+						}
+					}
+					Expect(len(sourceAttachmentPods) == 1).To(BeTrue())
+
+					migration := tests.NewRandomMigration(vmi.Name, vmi.Namespace)
+					migrationUID := tests.RunMigrationAndExpectCompletion(virtClient, migration, tests.MigrationWaitTime)
+					tests.ConfirmVMIPostMigration(virtClient, vmi, migrationUID)
+					By("Verifying the volume is still accessible and usable")
+					verifyVolumeAccessible(vmi, targets[0])
+					verifyWriteReadData(vmi, targets[0])
+
+					By("Verifying the source attachment pods are deleted")
+					Eventually(func() bool {
+						_, err := virtClient.CoreV1().Pods(vmi.Namespace).Get(context.Background(), sourceAttachmentPods[0], metav1.GetOptions{})
+						return errors.IsNotFound(err)
+					}, 60*time.Second, 1*time.Second).Should(BeTrue())
+				}
+
+				By("Verifying the volume can be detached and reattached after migration")
+				removeVolumeFunc(vmi.Name, vmi.Namespace, volumeName)
+				verifyVolumeNolongerAccessible(vmi, targets[0])
+				addVolumeFunc(vmi.Name, vmi.Namespace, volumeName, dv.Name, "scsi")
 				vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
-				By("Verifying the migration disabled reason is hotplug")
-				for _, condition := range vmi.Status.Conditions {
-					if condition.Type == kubevirtv1.VirtualMachineInstanceIsMigratable {
-						Expect(condition.Reason).To(Equal(kubevirtv1.VirtualMachineInstanceReasonHotplugNotMigratable))
-						break
-					}
-				}
-				removeVolumeFunc(vmi.Name, vmi.Namespace, "testvolume")
-				By("Verifying the VMI is migrateable")
-				verifyIsMigratable(vmi, true)
+				verifyVolumeAndDiskVMIAdded(vmi, volumeName)
+				verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, volumeName)
 			})
 		})
 	})
@@ -976,12 +1045,16 @@ var _ = SIGDescribe("Hotplug", func() {
 			}
 			vm = createVirtualMachine(true, template)
 			Eventually(func() bool {
-				vm, err := virtClient.VirtualMachine(tests.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
+				vm, err := virtClient.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				return vm.Status.Ready
 			}, 300*time.Second, 1*time.Second).Should(BeTrue())
 		}, 120)
 
+		AfterEach(func() {
+			tests.DeletePvAndPvc(fmt.Sprintf("%s-disk-for-tests", tests.CustomHostPath))
+		})
+
 		It("should attach a hostpath based volume to running VM", func() {
 			vmi, err := virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
@@ -1000,12 +1073,57 @@ var _ = SIGDescribe("Hotplug", func() {
 			getVmiConsoleAndLogin(vmi)
 			targets := getTargetsFromVolumeStatus(vmi, "testvolume")
 			verifyVolumeAccessible(vmi, targets[0])
+			verifySingleAttachmentPod(vmi)
+			By("removing volume from VM")
+			removeVolumeVMI(vm.Name, vm.Namespace, "testvolume")
+			verifyVolumeNolongerAccessible(vmi, targets[0])
+		})
+	})
+
+	Context("hostpath-separate-device", func() {
+		var (
+			vm *kubevirtv1.VirtualMachine
+		)
+
+		BeforeEach(func() {
+			tests.CreateAllSeparateDeviceHostPathPvs(tests.CustomHostPath)
+			vm = createVirtualMachine(true, libvmi.NewCirros())
+			Eventually(func() bool {
+				vm, err := virtClient.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				return vm.Status.Ready
+			}, 300*time.Second, 1*time.Second).Should(BeTrue())
+		}, 120)
+
+		AfterEach(func() {
+			tests.DeleteAllSeparateDeviceHostPathPvs()
+		})
+
+		It("should attach a hostpath based volume to running VM", func() {
+			dv := tests.NewRandomBlankDataVolume(util.NamespaceTestDefault, tests.Config.StorageClassHostPathSeparateDevice, "64Mi", corev1.ReadWriteOnce, corev1.PersistentVolumeFilesystem)
+			_, err := virtClient.CdiClient().CdiV1beta1().DataVolumes(dv.Namespace).Create(context.TODO(), dv, metav1.CreateOptions{})
+			Expect(err).To(BeNil())
+
+			vmi, err := virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			tests.WaitForSuccessfulVMIStartWithTimeout(vmi, 240)
+
+			By("Adding volume to running VM")
+			addPVCVolumeVMI(vm.Name, vm.Namespace, "testvolume", dv.Name, "scsi")
+
+			By("Verifying the volume and disk are in the VM and VMI")
+			vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			verifyVolumeAndDiskVMIAdded(vmi, "testvolume")
+			verifyVolumeStatus(vmi, kubevirtv1.VolumeReady, "testvolume")
+
+			getVmiConsoleAndLogin(vmi)
+			targets := getTargetsFromVolumeStatus(vmi, "testvolume")
+			verifyVolumeAccessible(vmi, targets[0])
+			verifySingleAttachmentPod(vmi)
 			By("removing volume from VM")
 			removeVolumeVMI(vm.Name, vm.Namespace, "testvolume")
 			verifyVolumeNolongerAccessible(vmi, targets[0])
-			By("Verifying the secret is gone")
-			_, err = virtClient.CoreV1().Secrets(vmi.Namespace).Get(context.Background(), name, metav1.GetOptions{})
-			Expect(err).To(HaveOccurred())
 		})
 	})
 
@@ -1022,11 +1140,11 @@ var _ = SIGDescribe("Hotplug", func() {
 			vmi, err := virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			tests.WaitForSuccessfulVMIStartWithTimeout(vmi, 240)
-			dv := tests.NewRandomBlankDataVolume(tests.NamespaceTestDefault, tests.Config.StorageClassLocal, "64Mi", corev1.ReadWriteOnce, corev1.PersistentVolumeFilesystem)
-			_, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(dv.Namespace).Create(context.TODO(), dv, metav1.CreateOptions{})
+			dv := tests.NewRandomBlankDataVolume(util.NamespaceTestDefault, tests.Config.StorageClassLocal, "64Mi", corev1.ReadWriteOnce, corev1.PersistentVolumeFilesystem)
+			_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(dv.Namespace).Create(context.TODO(), dv, metav1.CreateOptions{})
 			Expect(err).To(BeNil())
 			Eventually(func() error {
-				_, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(dv.Namespace).Get(context.TODO(), dv.Name, metav1.GetOptions{})
+				_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(dv.Namespace).Get(context.TODO(), dv.Name, metav1.GetOptions{})
 				return err
 			}, 40*time.Second, 2*time.Second).Should(Succeed())
 
@@ -1039,6 +1157,7 @@ var _ = SIGDescribe("Hotplug", func() {
 			getVmiConsoleAndLogin(vmi)
 			targets := getTargetsFromVolumeStatus(vmi, dv.Name)
 			verifyVolumeAccessible(vmi, targets[0])
+			verifySingleAttachmentPod(vmi)
 
 			By("Invoking virtlctl removevolume")
 			removeVolumeCommand := tests.NewRepeatableVirtctlCommand(virtctl.COMMAND_REMOVEVOLUME, vmi.Name, fmt.Sprintf(virtCtlVolumeName, dv.Name), virtCtlNamespace, vmi.Namespace)
